@@ -19,16 +19,24 @@
 #include "Scintilla.h"
 #include "SciLexer.h"
 
+#ifdef SCI_NAMESPACE
+using namespace Scintilla;
+#endif
+
 const LexerModule *LexerModule::base = 0;
 int LexerModule::nextLanguage = SCLEX_AUTOMATIC+1;
 
-LexerModule::LexerModule(int language_, LexerFunction fnLexer_,
-	const char *languageName_, LexerFunction fnFolder_,
-	const char * const wordListDescriptions_[]) :
+LexerModule::LexerModule(int language_,
+	LexerFunction fnLexer_,
+	const char *languageName_,
+	LexerFunction fnFolder_,
+	const char * const wordListDescriptions_[],
+	int styleBits_) :
 	language(language_),
 	fnLexer(fnLexer_),
 	fnFolder(fnFolder_),
 	wordListDescriptions(wordListDescriptions_),
+	styleBits(styleBits_),
 	languageName(languageName_) {
 	next = base;
 	base = this;
@@ -52,7 +60,7 @@ int LexerModule::GetNumWordLists() const {
 	}
 }
 
-const char * LexerModule::GetWordListDescription(int index) const {
+const char *LexerModule::GetWordListDescription(int index) const {
 	static const char *emptyStr = "";
 
 	PLATFORM_ASSERT(index < GetNumWordLists());
@@ -63,6 +71,10 @@ const char * LexerModule::GetWordListDescription(int index) const {
  	}
 }
 
+int LexerModule::GetStyleBitsNeeded() const {
+	return styleBits;
+}
+
 const LexerModule *LexerModule::Find(int language) {
 	const LexerModule *lm = base;
 	while (lm) {
@@ -129,78 +141,92 @@ int Scintilla_LinkLexers() {
 
 //++Autogenerated -- run src/LexGen.py to regenerate
 //**\(\tLINK_LEXER(\*);\n\)
+	LINK_LEXER(lmAbaqus);
 	LINK_LEXER(lmAda);
+	LINK_LEXER(lmAns1);
 	LINK_LEXER(lmAPDL);
 	LINK_LEXER(lmAsm);
-	LINK_LEXER(lmAns1);
+	LINK_LEXER(lmASP);
+	LINK_LEXER(lmASY);
 	LINK_LEXER(lmAU3);
 	LINK_LEXER(lmAVE);
 	LINK_LEXER(lmBaan);
 	LINK_LEXER(lmBash);
+	LINK_LEXER(lmBatch);
 	LINK_LEXER(lmBlitzBasic);
-	LINK_LEXER(lmPureBasic);
 	LINK_LEXER(lmBullant);
 	LINK_LEXER(lmCaml);
 	LINK_LEXER(lmClw);
 	LINK_LEXER(lmClwNoCase);
+	LINK_LEXER(lmCmake);
 	LINK_LEXER(lmConf);
 	LINK_LEXER(lmCPP);
 	LINK_LEXER(lmCPPNoCase);
-	LINK_LEXER(lmTCL);
-	LINK_LEXER(lmNncrontab);
 	LINK_LEXER(lmCsound);
 	LINK_LEXER(lmCss);
+	LINK_LEXER(lmD);
+	LINK_LEXER(lmDiff);
 	LINK_LEXER(lmEiffel);
 	LINK_LEXER(lmEiffelkw);
 	LINK_LEXER(lmErlang);
+	LINK_LEXER(lmErrorList);
 	LINK_LEXER(lmESCRIPT);
+	LINK_LEXER(lmF77);
 	LINK_LEXER(lmFlagShip);
 	LINK_LEXER(lmForth);
 	LINK_LEXER(lmFortran);
-	LINK_LEXER(lmF77);
+	LINK_LEXER(lmFreeBasic);
+	LINK_LEXER(lmGAP);
 	LINK_LEXER(lmGui4Cli);
 	LINK_LEXER(lmHaskell);
 	LINK_LEXER(lmHTML);
-	LINK_LEXER(lmXML);
-	LINK_LEXER(lmASP);
-	LINK_LEXER(lmPHP);
-	LINK_LEXER(lmPHPSCRIPT);
+	LINK_LEXER(lmInno);
 	LINK_LEXER(lmKix);
+	LINK_LEXER(lmLatex);
 	LINK_LEXER(lmLISP);
+	LINK_LEXER(lmLot);
 	LINK_LEXER(lmLout);
 	LINK_LEXER(lmLua);
+	LINK_LEXER(lmMagikSF);
+	LINK_LEXER(lmMake);
 	LINK_LEXER(lmMatlab);
-	LINK_LEXER(lmOctave);
 	LINK_LEXER(lmMETAPOST);
 	LINK_LEXER(lmMMIXAL);
-	LINK_LEXER(lmLot);
 	LINK_LEXER(lmMSSQL);
+	LINK_LEXER(lmNncrontab);
 	LINK_LEXER(lmNsis);
-	LINK_LEXER(lmBatch);
-	LINK_LEXER(lmDiff);
-	LINK_LEXER(lmProps);
-	LINK_LEXER(lmMake);
-	LINK_LEXER(lmErrorList);
-	LINK_LEXER(lmLatex);
 	LINK_LEXER(lmNull);
+	LINK_LEXER(lmOctave);
+	LINK_LEXER(lmOpal);
 	LINK_LEXER(lmPascal);
 	LINK_LEXER(lmPB);
 	LINK_LEXER(lmPerl);
+	LINK_LEXER(lmPHP);
+	LINK_LEXER(lmPHPSCRIPT);
+	LINK_LEXER(lmPLM);
 	LINK_LEXER(lmPOV);
+	LINK_LEXER(lmPowerShell);
+	LINK_LEXER(lmProgress);
+	LINK_LEXER(lmProps);
 	LINK_LEXER(lmPS);
+	LINK_LEXER(lmPureBasic);
 	LINK_LEXER(lmPython);
+	LINK_LEXER(lmR);
 	LINK_LEXER(lmREBOL);
 	LINK_LEXER(lmRuby);
 	LINK_LEXER(lmScriptol);
 	LINK_LEXER(lmSmalltalk);
 	LINK_LEXER(lmSpecman);
+	LINK_LEXER(lmSpice);
 	LINK_LEXER(lmSQL);
 	LINK_LEXER(lmTADS3);
+	LINK_LEXER(lmTCL);
 	LINK_LEXER(lmTeX);
 	LINK_LEXER(lmVB);
 	LINK_LEXER(lmVBScript);
 	LINK_LEXER(lmVerilog);
 	LINK_LEXER(lmVHDL);
+	LINK_LEXER(lmXML);
 	LINK_LEXER(lmYAML);
 
 //--Autogenerated -- end of automatically generated section
