@@ -2,7 +2,7 @@
 /** @file LexBash.cxx
  ** Lexer for Bash.
  **/
-// Copyright 2004 by Neil Hodgson <neilh@scintilla.org>
+// Copyright 2004-2007 by Neil Hodgson <neilh@scintilla.org>
 // Adapted from LexPerl by Kein-Hong Man <mkh@pl.jaring.my> 2004
 // The License.txt file describes the conditions under which this software may be distributed.
 
@@ -20,14 +20,24 @@
 #include "Scintilla.h"
 #include "SciLexer.h"
 
+// define this if you want 'invalid octals' to be marked as errors
+// usually, this is not a good idea, permissive lexing is better
+#undef PEDANTIC_OCTAL
+
 #define BASH_BASE_ERROR		65
 #define BASH_BASE_DECIMAL	66
 #define BASH_BASE_HEX		67
+#ifdef PEDANTIC_OCTAL
 #define BASH_BASE_OCTAL		68
 #define BASH_BASE_OCTAL_ERROR	69
+#endif
 
 #define HERE_DELIM_MAX 256
 
+#ifdef SCI_NAMESPACE
+using namespace Scintilla;
+#endif
+
 static inline int translateBashDigit(char ch) {
 	if (ch >= '0' && ch <= '9') {
 		return ch - '0';
@@ -144,6 +154,9 @@ static void ColouriseBashDoc(unsigned int startPos, int length, int initStyle,
 		char *Delimiter;	// the Delimiter, 256: sizeof PL_tokenbuf
 		HereDocCls() {
 			State = 0;
+            Quote = 0;
+            Quoted = false;
+            Indent = 0;
 			DelimiterLength = 0;
 			Delimiter = new char[HERE_DELIM_MAX];
 			Delimiter[0] = '\0';
@@ -255,7 +268,8 @@ static void ColouriseBashDoc(unsigned int startPos, int length, int initStyle,
 
 		if (state == SCE_SH_DEFAULT) {
 			if (ch == '\\') {	// escaped character
-				i++;
+				if (i < lengthDoc - 1)
+					i++;
 				ch = chNext;
 				chNext = chNext2;
 				styler.ColourTo(i, SCE_SH_IDENTIFIER);
@@ -269,7 +283,11 @@ static void ColouriseBashDoc(unsigned int startPos, int length, int initStyle,
 						ch = chNext;
 						chNext = chNext2;
 					} else if (isdigit(chNext)) {
+#ifdef PEDANTIC_OCTAL
 						numBase = BASH_BASE_OCTAL;
+#else
+						numBase = BASH_BASE_HEX;
+#endif
 					}
 				}
 			} else if (iswordstart(ch)) {
@@ -334,7 +352,8 @@ static void ColouriseBashDoc(unsigned int startPos, int length, int initStyle,
 				HereDoc.Indent = false;
 			} else if (ch == '-'	// file test operators
 			           && isSingleCharOp(chNext)
-			           && !isalnum((chNext2 = styler.SafeGetCharAt(i+2)))) {
+			           && !isalnum((chNext2 = styler.SafeGetCharAt(i+2)))
+			           && isspace(chPrev)) {
 				styler.ColourTo(i + 1, SCE_SH_WORD);
 				state = SCE_SH_DEFAULT;
 				i++;
@@ -360,14 +379,16 @@ static void ColouriseBashDoc(unsigned int startPos, int length, int initStyle,
 					// hex digit 0-9a-fA-F
 				} else
 					goto numAtEnd;
+#ifdef PEDANTIC_OCTAL
 			} else if (numBase == BASH_BASE_OCTAL ||
 				   numBase == BASH_BASE_OCTAL_ERROR) {
 				if (digit > 7) {
 					if (digit <= 9) {
-						numBase = BASH_BASE_OCTAL_ERROR;
+                                                numBase = BASH_BASE_OCTAL_ERROR;
 					} else
 						goto numAtEnd;
 				}
+#endif
 			} else if (numBase == BASH_BASE_ERROR) {
 				if (digit > 9)
 					goto numAtEnd;
@@ -385,8 +406,11 @@ static void ColouriseBashDoc(unsigned int startPos, int length, int initStyle,
 					}
 				} else {
 			numAtEnd:
-					if (numBase == BASH_BASE_ERROR ||
-					    numBase == BASH_BASE_OCTAL_ERROR)
+					if (numBase == BASH_BASE_ERROR
+#ifdef PEDANTIC_OCTAL
+					    || numBase == BASH_BASE_OCTAL_ERROR
+#endif
+                                           )
 						state = SCE_SH_ERROR;
 					styler.ColourTo(i - 1, state);
 					state = SCE_SH_DEFAULT;
@@ -442,7 +466,7 @@ static void ColouriseBashDoc(unsigned int startPos, int length, int initStyle,
 					HereDoc.Quoted = false;
 					HereDoc.DelimiterLength = 0;
 					HereDoc.Delimiter[HereDoc.DelimiterLength] = '\0';
-					if (chNext == '\'') {	// a quoted here-doc delimiter (' only)
+					if (chNext == '\'' || chNext == '\"') {	// a quoted here-doc delimiter (' or ")
 						i++;
 						ch = chNext;
 						chNext = chNext2;
@@ -451,8 +475,9 @@ static void ColouriseBashDoc(unsigned int startPos, int length, int initStyle,
 						HereDoc.Indent = true;
 						HereDoc.State = 0;
 					} else if (isalpha(chNext) || chNext == '_' || chNext == '\\'
-						|| chNext == '-' || chNext == '+') {
+						|| chNext == '-' || chNext == '+' || chNext == '!') {
 						// an unquoted here-doc delimiter, no special handling
+                        // TODO check what exactly bash considers part of the delim
 					} else if (chNext == '<') {	// HERE string <<<
 						i++;
 						ch = chNext;
@@ -486,7 +511,7 @@ static void ColouriseBashDoc(unsigned int startPos, int length, int initStyle,
 							HereDoc.Delimiter[HereDoc.DelimiterLength] = '\0';
 						}
 					} else { // an unquoted here-doc delimiter
-						if (isalnum(ch) || ch == '_' || ch == '-' || ch == '+') {
+						if (isalnum(ch) || ch == '_' || ch == '-' || ch == '+' || ch == '!') {
 							HereDoc.Delimiter[HereDoc.DelimiterLength++] = ch;
 							HereDoc.Delimiter[HereDoc.DelimiterLength] = '\0';
 						} else if (ch == '\\') {
@@ -582,6 +607,19 @@ static void ColouriseBashDoc(unsigned int startPos, int length, int initStyle,
 	styler.ColourTo(lengthDoc - 1, state);
 }
 
+static bool IsCommentLine(int line, Accessor &styler) {
+	int pos = styler.LineStart(line);
+	int eol_pos = styler.LineStart(line + 1) - 1;
+	for (int i = pos; i < eol_pos; i++) {
+		char ch = styler[i];
+		if (ch == '#')
+			return true;
+		else if (ch != ' ' && ch != '\t')
+			return false;
+	}
+	return false;
+}
+
 static void FoldBashDoc(unsigned int startPos, int length, int, WordList *[],
                             Accessor &styler) {
 	bool foldComment = styler.GetPropertyInt("fold.comment") != 0;
@@ -599,17 +637,17 @@ static void FoldBashDoc(unsigned int startPos, int length, int, WordList *[],
 		int style = styleNext;
 		styleNext = styler.StyleAt(i + 1);
 		bool atEOL = (ch == '\r' && chNext != '\n') || (ch == '\n');
-		if (foldComment && (style == SCE_SH_COMMENTLINE)) {
-			if ((ch == '/') && (chNext == '/')) {
-				char chNext2 = styler.SafeGetCharAt(i + 2);
-				if (chNext2 == '{') {
-					levelCurrent++;
-				} else if (chNext2 == '}') {
-					levelCurrent--;
-				}
-			}
-		}
-		if (style == SCE_C_OPERATOR) {
+        // Comment folding
+		if (foldComment && atEOL && IsCommentLine(lineCurrent, styler))
+        {
+            if (!IsCommentLine(lineCurrent - 1, styler)
+                && IsCommentLine(lineCurrent + 1, styler))
+                levelCurrent++;
+            else if (IsCommentLine(lineCurrent - 1, styler)
+                     && !IsCommentLine(lineCurrent+1, styler))
+                levelCurrent--;
+        }
+		if (style == SCE_SH_OPERATOR) {
 			if (ch == '{') {
 				levelCurrent++;
 			} else if (ch == '}') {
