@@ -1,5 +1,5 @@
 /*
-** $Id: lzio.h,v 1.1 2004/03/27 00:19:45 nyamatongwe Exp $
+** $Id: lzio.h,v 1.2 2007/06/15 00:37:57 nyamatongwe Exp $
 ** Buffered streams
 ** See Copyright Notice in lua.h
 */
@@ -10,36 +10,31 @@
 
 #include "lua.h"
 
+#include "lmem.h"
+
 
 #define EOZ	(-1)			/* end of stream */
 
 typedef struct Zio ZIO;
 
-
 #define char2int(c)	cast(int, cast(unsigned char, (c)))
 
 #define zgetc(z)  (((z)->n--)>0 ?  char2int(*(z)->p++) : luaZ_fill(z))
 
-#define zname(z)	((z)->name)
-
-void luaZ_init (ZIO *z, lua_Chunkreader reader, void *data, const char *name);
-size_t luaZ_read (ZIO* z, void* b, size_t n);	/* read next n bytes */
-int luaZ_lookahead (ZIO *z);
-
-
-
 typedef struct Mbuffer {
   char *buffer;
+  size_t n;
   size_t buffsize;
 } Mbuffer;
 
-
-char *luaZ_openspace (lua_State *L, Mbuffer *buff, size_t n);
-
 #define luaZ_initbuffer(L, buff) ((buff)->buffer = NULL, (buff)->buffsize = 0)
 
-#define luaZ_sizebuffer(buff)	((buff)->buffsize)
 #define luaZ_buffer(buff)	((buff)->buffer)
+#define luaZ_sizebuffer(buff)	((buff)->buffsize)
+#define luaZ_bufflen(buff)	((buff)->n)
+
+#define luaZ_resetbuffer(buff) ((buff)->n = 0)
+
 
 #define luaZ_resizebuffer(L, buff, size) \
 	(luaM_reallocvector(L, (buff)->buffer, (buff)->buffsize, size, char), \
@@ -48,17 +43,25 @@ char *luaZ_openspace (lua_State *L, Mbuffer *buff, size_t n);
 #define luaZ_freebuffer(L, buff)	luaZ_resizebuffer(L, buff, 0)
 
 
+LUAI_FUNC char *luaZ_openspace (lua_State *L, Mbuffer *buff, size_t n);
+LUAI_FUNC void luaZ_init (lua_State *L, ZIO *z, lua_Reader reader,
+                                        void *data);
+LUAI_FUNC size_t luaZ_read (ZIO* z, void* b, size_t n);	/* read next n bytes */
+LUAI_FUNC int luaZ_lookahead (ZIO *z);
+
+
+
 /* --------- Private Part ------------------ */
 
 struct Zio {
   size_t n;			/* bytes still unread */
   const char *p;		/* current position in buffer */
-  lua_Chunkreader reader;
+  lua_Reader reader;
   void* data;			/* additional data */
-  const char *name;
+  lua_State *L;			/* Lua state (for reader) */
 };
 
 
-int luaZ_fill (ZIO *z);
+LUAI_FUNC int luaZ_fill (ZIO *z);
 
 #endif
