@@ -48,12 +48,15 @@
 
 #include "SciTE.h"
 #include "PropSet.h"
+#include "StringList.h"
 #include "Accessor.h"
 #include "WindowAccessor.h"
 #include "Scintilla.h"
 #include "Extender.h"
 #include "FilePath.h"
 #include "PropSetFile.h"
+#include "Mutex.h"
+#include "JobQueue.h"
 #include "SciTEBase.h"
 
 
@@ -299,8 +302,7 @@ void SciTEBase::SaveToRTF(FilePath saveName, int start, int end) {
 			} else if (ch == '\r') {
 				fputs(RTF_EOLN, fp);
 				column = -1;
-			}
-			else
+			} else
 				fputc(ch, fp);
 			column++;
 			prevCR = ch == '\r';
@@ -355,25 +357,37 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 		fputs("<head>\n", fp);
 		if (titleFullPath)
 			fprintf(fp, "<title>%s</title>\n",
-				static_cast<const char *>(filePath.AsFileSystem()));
+			        static_cast<const char *>(filePath.AsFileSystem()));
 		else
 			fprintf(fp, "<title>%s</title>\n",
-				static_cast<const char *>(filePath.Name().AsFileSystem()));
+			        static_cast<const char *>(filePath.Name().AsFileSystem()));
 		// Probably not used by robots, but making a little advertisement for those looking
 		// at the source code doesn't hurt...
 		fputs("<meta name=\"Generator\" content=\"SciTE - www.Scintilla.org\" />\n", fp);
-                if (codePage == SC_CP_UTF8)
-                        fputs("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n", fp);
+		if (codePage == SC_CP_UTF8)
+			fputs("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n", fp);
 
 		if (folding) {
 			fputs("<script language=\"JavaScript\" type=\"text/javascript\">\n"
 			      "<!--\n"
-			      "function toggle(thisid) {\n"
-			      "var thislayer=document.getElementById(thisid);\n"
+			      "function symbol(id, sym) {\n"
+			      " if (id.textContent==undefined) {\n"
+			      " id.innerText=sym; } else {\n"
+			      " id.textContent=sym; }\n"
+			      "}\n"
+			      "function toggle(id) {\n"
+			      "var thislayer=document.getElementById('ln'+id);\n"
+			      "id-=1;\n"
+			      "var togline=document.getElementById('hd'+id);\n"
+			      "var togsym=document.getElementById('bt'+id);\n"
 			      "if (thislayer.style.display == 'none') {\n"
 			      " thislayer.style.display='block';\n"
+			      " togline.style.textDecoration='none';\n"
+			      " symbol(togsym,'- ');\n"
 			      "} else {\n"
 			      " thislayer.style.display='none';\n"
+			      " togline.style.textDecoration='underline';\n"
+			      " symbol(togsym,'+ ');\n"
 			      "}\n"
 			      "}\n"
 			      "//-->\n"
@@ -383,15 +397,32 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 		fputs("<style type=\"text/css\">\n", fp);
 
 		SString bgColour;
+		char key[200];
+		sprintf(key, "style.*.%0d", STYLE_DEFAULT);
+		char *valdef = StringDup(props.GetExpanded(key).c_str());
+		sprintf(key, "style.%s.%0d", language.c_str(), STYLE_DEFAULT);
+		char *val = StringDup(props.GetExpanded(key).c_str());
+
+		StyleDefinition sddef(valdef);
+		sddef.ParseStyleDefinition(val);
+		if (sddef.back.length()) {
+			bgColour = sddef.back;
+		}
+		if (val) {
+			delete []val;
+		}
+		if (valdef) {
+			delete []valdef;
+		}
+
 		for (int istyle = 0; istyle <= STYLE_MAX; istyle++) {
 			if ((istyle > STYLE_DEFAULT) && (istyle <= STYLE_LASTPREDEFINED))
 				continue;
 			if (styleIsUsed[istyle]) {
-				char key[200];
 				sprintf(key, "style.*.%0d", istyle);
-				char *valdef = StringDup(props.GetExpanded(key).c_str());
+				valdef = StringDup(props.GetExpanded(key).c_str());
 				sprintf(key, "style.%s.%0d", language.c_str(), istyle);
-				char *val = StringDup(props.GetExpanded(key).c_str());
+				val = StringDup(props.GetExpanded(key).c_str());
 
 				StyleDefinition sd(valdef);
 				sd.ParseStyleDefinition(val);
@@ -417,9 +448,10 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 						fprintf(fp, "\tcolor: #000000;\n");
 					}
 					if (sd.back.length()) {
-						fprintf(fp, "\tbackground: %s;\n", sd.back.c_str());
-						if (istyle == STYLE_DEFAULT)
-							bgColour = sd.back;
+						if (istyle != STYLE_DEFAULT && bgColour != sd.back) {
+							fprintf(fp, "\tbackground: %s;\n", sd.back.c_str());
+							fprintf(fp, "\ttext-decoration: inherit;\n");
+						}
 					}
 					if (wysiwyg && sd.size) {
 						fprintf(fp, "\tfont-size: %0dpt;\n", sd.size);
@@ -449,6 +481,7 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 		int newLevel;
 		int styleCurrent = acc.StyleAt(0);
 		bool inStyleSpan = false;
+		bool inFoldSpan = false;
 		// Global span for default attributes
 		if (wysiwyg) {
 			fputs("<span>", fp);
@@ -461,7 +494,9 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 			level = (lvl & SC_FOLDLEVELNUMBERMASK) - SC_FOLDLEVELBASE;
 
 			if (lvl & SC_FOLDLEVELHEADERFLAG) {
-				fprintf(fp, "<span onclick=\"toggle('ln%d')\">-</span> ", line + 1);
+				fprintf(fp, "<span id=\"hd%d\" onclick=\"toggle('%d')\">", line, line + 1);
+				fprintf(fp, "<span id=\"bt%d\">- </span>", line);
+				inFoldSpan = true;
 			} else {
 				fputs("&nbsp; ", fp);
 			}
@@ -540,6 +575,10 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 					fputs("</span>", fp);
 					inStyleSpan = false;
 				}
+				if (inFoldSpan) {
+					fputs("</span>", fp);
+					inFoldSpan = false;
+				}
 				if (ch == '\r' && acc[i + 1] == '\n') {
 					i++;	// CR+LF line ending, skip the "extra" EOL char
 				}
@@ -561,9 +600,11 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 					if (newLevel > level)
 						fprintf(fp, "<span id=\"ln%d\">", line);
 
-					if (lvl & SC_FOLDLEVELHEADERFLAG)
-						fprintf(fp, "<span onclick=\"toggle('ln%d')\">-</span> ", line + 1);
-					else
+					if (lvl & SC_FOLDLEVELHEADERFLAG) {
+						fprintf(fp, "<span id=\"hd%d\" onclick=\"toggle('%d')\">", line, line + 1);
+						fprintf(fp, "<span id=\"bt%d\">- </span>", line);
+						inFoldSpan = true;
+					} else
 						fputs("&nbsp; ", fp);
 					level = newLevel;
 				} else {
@@ -615,7 +656,7 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 		fclose(fp);
 	} else {
 		SString msg = LocaliseMessage(
-		                  "Could not save file \"^0\".", filePath.AsFileSystem());
+		            "Could not save file \"^0\".", filePath.AsFileSystem());
 		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
 	}
 }
@@ -652,10 +693,10 @@ struct PDFStyle {
 };
 
 static char *PDFfontNames[] = {
-	"Courier", "Courier-Bold", "Courier-Oblique", "Courier-BoldOblique",
-	"Helvetica", "Helvetica-Bold", "Helvetica-Oblique", "Helvetica-BoldOblique",
-	"Times-Roman", "Times-Bold", "Times-Italic", "Times-BoldItalic"
-};
+            "Courier", "Courier-Bold", "Courier-Oblique", "Courier-BoldOblique",
+            "Helvetica", "Helvetica-Bold", "Helvetica-Oblique", "Helvetica-BoldOblique",
+            "Times-Roman", "Times-Bold", "Times-Italic", "Times-BoldItalic"
+        };
 
 // ascender, descender aligns font origin point with page
 static short PDFfontAscenders[] =  { 629, 718, 699 };
@@ -792,12 +833,12 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 			*buff = '\0';
 			if (styleNext != styleCurrent || style_ == -1) {
 				if (style[styleCurrent].font != style[styleNext].font
-				    || style_ == -1) {
+				        || style_ == -1) {
 					sprintf(buff, "/F%d %d Tf ",
-						style[styleNext].font + 1, fontSize);
+					        style[styleNext].font + 1, fontSize);
 				}
 				if (strcmp(style[styleCurrent].fore, style[styleNext].fore) != 0
-				    || style_ == -1) {
+				        || style_ == -1) {
 					strcat(buff, style[styleNext].fore);
 					strcat(buff, "rg ");
 				}
@@ -806,7 +847,7 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 		//
 		void startPDF() {
 			if (fontSize <= 0) {
-			    fontSize = PDF_FONTSIZE_DEFAULT;
+				fontSize = PDF_FONTSIZE_DEFAULT;
 			}
 			// leading is the term for distance between lines
 			leading = fontSize * PDF_SPACING_DEFAULT;
@@ -829,10 +870,10 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 			// to be inserted (PDF1.4Ref(p317))
 			for (int i = 0; i < 4; i++) {
 				sprintf(buffer, "<</Type/Font/Subtype/Type1"
-						"/Name/F%d/BaseFont/%s/Encoding/"
-						PDF_ENCODING
-						">>\n", i + 1,
-						PDFfontNames[fontSet * 4 + i]);
+				        "/Name/F%d/BaseFont/%s/Encoding/"
+				        PDF_ENCODING
+				        ">>\n", i + 1,
+				        PDFfontNames[fontSet * 4 + i]);
 				oT->add(buffer);
 			}
 			pageContentStart = oT->index;
@@ -843,20 +884,20 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 			}
 			// refer to all used or unused fonts for simplicity
 			int resourceRef = oT->add(
-				"<</ProcSet[/PDF/Text]\n"
-				"/Font<</F1 1 0 R/F2 2 0 R/F3 3 0 R"
-				"/F4 4 0 R>> >>\n");
+			            "<</ProcSet[/PDF/Text]\n"
+			            "/Font<</F1 1 0 R/F2 2 0 R/F3 3 0 R"
+			            "/F4 4 0 R>> >>\n");
 			// create all the page objects (PDF1.4Ref(p88))
 			// forward reference pages object; calculate its object number
 			int pageObjectStart = oT->index;
 			int pagesRef = pageObjectStart + pageCount;
 			for (int i = 0; i < pageCount; i++) {
 				sprintf(buffer, "<</Type/Page/Parent %d 0 R\n"
-						"/MediaBox[ 0 0 %d %d"
-						"]\n/Contents %d 0 R\n"
-						"/Resources %d 0 R\n>>\n",
-						pagesRef, pageWidth, pageHeight,
-						pageContentStart + i, resourceRef);
+				        "/MediaBox[ 0 0 %d %d"
+				        "]\n/Contents %d 0 R\n"
+				        "/Resources %d 0 R\n>>\n",
+				        pagesRef, pageWidth, pageHeight,
+				        pageContentStart + i, resourceRef);
 				oT->add(buffer);
 			}
 			// create page tree object (PDF1.4Ref(p86))
@@ -875,8 +916,8 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 			int xref = oT->xref();
 			// end the file with the trailer (PDF1.4Ref(p67))
 			sprintf(buffer, "trailer\n<< /Size %d /Root %d 0 R\n>>"
-					"\nstartxref\n%d\n%%%%EOF\n",
-					oT->index, catalogRef, xref);
+			        "\nstartxref\n%d\n%%%%EOF\n",
+			        oT->index, catalogRef, xref);
 			oT->write(buffer);
 		}
 		void add(char ch, int style_) {
@@ -929,7 +970,7 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 			yPos = pageHeight - pageMargin.top - fontAscender;
 			// start a new page
 			sprintf(buffer, "BT 1 0 0 1 %d %d Tm\n",
-				pageMargin.left, (int)yPos);
+			        pageMargin.left, (int)yPos);
 			// force setting of initial font, colour
 			setStyle(segStyle, -1);
 			strcat(buffer, segStyle);
@@ -946,9 +987,9 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 			char *textObj = new char[pageData.length() + 100];
 			// concatenate stream within the text object
 			sprintf(textObj, "<</Length %d>>\nstream\n%s"
-					 "ET\nendstream\n",
-					 static_cast<int>(pageData.length() - 1 + 3),
-					 pageData.c_str());
+			        "ET\nendstream\n",
+			        static_cast<int>(pageData.length() - 1 + 3),
+			        pageData.c_str());
 			oT->add(textObj);
 			delete []textObj;
 		}
@@ -967,7 +1008,9 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 				return;
 			}
 			if (firstLine) {
-				sprintf(buffer, "0 -%.1f TD\n", leading);
+				// avoid breakage due to locale setting
+				int f = (int)(leading * 10 + 0.5);
+				sprintf(buffer, "0 -%d.%d TD\n", f / 10, f % 10);
 				firstLine = false;
 			} else {
 				sprintf(buffer, "T*\n");
@@ -1109,12 +1152,12 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 					i++;
 				}
 				// close and begin a newline...
-					pr.nextLine();
-					lineIndex = 0;
+				pr.nextLine();
+				lineIndex = 0;
 			} else {
 				// write the character normally...
-					pr.add(ch, style);
-					lineIndex++;
+				pr.add(ch, style);
+				lineIndex++;
 			}
 		}
 	}
@@ -1128,10 +1171,14 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 
 static char* getTexRGB(char* texcolor, const char* stylecolor) {
 	//texcolor[rgb]{0,0.5,0}{....}
-	float r = IntFromHexByte(stylecolor + 1) / 256.0;
-	float g = IntFromHexByte(stylecolor + 3) / 256.0;
-	float b = IntFromHexByte(stylecolor + 5) / 256.0;
-	sprintf(texcolor, "%.1f, %.1f, %.1f", r, g, b);
+	double rf = IntFromHexByte(stylecolor + 1) / 256.0;
+	double gf = IntFromHexByte(stylecolor + 3) / 256.0;
+	double bf = IntFromHexByte(stylecolor + 5) / 256.0;
+	// avoid breakage due to locale setting
+	int r = (int)(rf * 10 + 0.5);
+	int g = (int)(gf * 10 + 0.5);
+	int b = (int)(bf * 10 + 0.5);
+	sprintf(texcolor, "%d.%d, %d.%d, %d.%d", r / 10, r % 10, g / 10, g % 10, b / 10, b % 10);
 	return texcolor;
 }
 
@@ -1228,7 +1275,7 @@ void SciTEBase::SaveToTEX(FilePath saveName) {
 
 		fputs("\\begin{document}\n\n", fp);
 		fprintf(fp, "Source File: %s\n\n\\noindent\n\\tiny{\n",
-			static_cast<const char *>(titleFullPath ? filePath.AsFileSystem() : filePath.Name().AsFileSystem()));
+		        static_cast<const char *>(titleFullPath ? filePath.AsFileSystem() : filePath.Name().AsFileSystem()));
 
 		int styleCurrent = acc.StyleAt(0);
 
@@ -1295,7 +1342,7 @@ void SciTEBase::SaveToTEX(FilePath saveName) {
 		fclose(fp);
 	} else {
 		SString msg = LocaliseMessage(
-		                  "Could not save file \"^0\".", filePath.AsFileSystem());
+		            "Could not save file \"^0\".", filePath.AsFileSystem());
 		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
 	}
 }
@@ -1305,176 +1352,176 @@ void SciTEBase::SaveToTEX(FilePath saveName) {
 
 void SciTEBase::SaveToXML(FilePath saveName) {
 
-    // Author: Hans Hagen / PRAGMA ADE / www.pragma-ade.com
-    // Version: 1.0 / august 18, 2003
-    // Remark: for a suitable style, see ConTeXt (future) distributions
+	// Author: Hans Hagen / PRAGMA ADE / www.pragma-ade.com
+	// Version: 1.0 / august 18, 2003
+	// Remark: for a suitable style, see ConTeXt (future) distributions
 
-    // The idea is that one can use whole files, or ranges of lines in manuals
-    // and alike. Since ConTeXt can handle XML files, it's quite convenient to
-    // use this format instead of raw TeX, although the output would not look
-    // much different in structure.
+	// The idea is that one can use whole files, or ranges of lines in manuals
+	// and alike. Since ConTeXt can handle XML files, it's quite convenient to
+	// use this format instead of raw TeX, although the output would not look
+	// much different in structure.
 
-    // We don't put style definitions in here since the main document will in
-    // most cases determine the look and feel. This way we have full control over
-    // the layout. The type attribute will hold the current lexer value.
+	// We don't put style definitions in here since the main document will in
+	// most cases determine the look and feel. This way we have full control over
+	// the layout. The type attribute will hold the current lexer value.
 
-    // <document>            : the whole thing
-    // <data>                : reserved for metadata
-    // <text>                : the main bodyof text
-    // <line n-'number'>     : a line of text
+	// <document>            : the whole thing
+	// <data>                : reserved for metadata
+	// <text>                : the main bodyof text
+	// <line n-'number'>     : a line of text
 
-    // <t n='number'>...<t/> : tag
-    // <s n='number'/>       : space
-    // <g/>                  : >
-    // <l/>                  : <
-    // <a/>                  : &
-    // <h/>                  : #
+	// <t n='number'>...<t/> : tag
+	// <s n='number'/>       : space
+	// <g/>                  : >
+	// <l/>                  : <
+	// <a/>                  : &
+	// <h/>                  : #
 
-    // We don't use entities, but empty elements for special characters
-    // but will eventually use utf-8 (once i know how to get them out).
+	// We don't use entities, but empty elements for special characters
+	// but will eventually use utf-8 (once i know how to get them out).
 
 	RemoveFindMarks();
-	SendEditor(SCI_COLOURISE, 0, -1) ;
+	SendEditor(SCI_COLOURISE, 0, -1);
 
-	int tabSize = props.GetInt("tabsize") ;
+	int tabSize = props.GetInt("tabsize");
 	if (tabSize == 0) {
-		tabSize = 4 ;
-    }
+		tabSize = 4;
+	}
 
-	int lengthDoc = LengthDocument() ;
+	int lengthDoc = LengthDocument();
 
-	WindowAccessor acc(wEditor.GetID(), props) ;
+	WindowAccessor acc(wEditor.GetID(), props);
 
 	FILE *fp = saveName.Open("wt");
 
 	if (fp) {
 
-        bool collapseSpaces = (props.GetInt("export.xml.collapse.spaces", 1) == 1) ;
-        bool collapseLines  = (props.GetInt("export.xml.collapse.lines", 1) == 1) ;
+		bool collapseSpaces = (props.GetInt("export.xml.collapse.spaces", 1) == 1);
+		bool collapseLines  = (props.GetInt("export.xml.collapse.lines", 1) == 1);
 
-		fputs("<?xml version='1.0' encoding='ascii'?>\n", fp) ;
+		fputs("<?xml version='1.0' encoding='ascii'?>\n", fp);
 
-		fputs("<document xmlns='http://www.scintila.org/scite.rng'", fp) ;
+		fputs("<document xmlns='http://www.scintila.org/scite.rng'", fp);
 		fprintf(fp, " filename='%s'",
-			static_cast<const char *>(filePath.Name().AsFileSystem())) ;
-		fprintf(fp, " type='%s'", "unknown") ;
-		fprintf(fp, " version='%s'", "1.0") ;
-        fputs(">\n", fp) ;
+		        static_cast<const char *>(filePath.Name().AsFileSystem()));
+		fprintf(fp, " type='%s'", "unknown");
+		fprintf(fp, " version='%s'", "1.0");
+		fputs(">\n", fp);
 
-        fputs("<data comment='This element is reserved for future usage.'/>\n", fp) ;
+		fputs("<data comment='This element is reserved for future usage.'/>\n", fp);
 
-		fputs("<text>\n", fp) ;
+		fputs("<text>\n", fp);
 
-		int styleCurrent = -1 ; // acc.StyleAt(0) ;
-        int lineNumber = 1 ;
-		int lineIndex = 0 ;
-        bool styleDone = false ;
-        bool lineDone = false ;
-        bool charDone = false ;
-        int styleNew = -1 ;
-        int spaceLen = 0 ;
-        int emptyLines = 0 ;
+		int styleCurrent = -1; // acc.StyleAt(0);
+		int lineNumber = 1;
+		int lineIndex = 0;
+		bool styleDone = false;
+		bool lineDone = false;
+		bool charDone = false;
+		int styleNew = -1;
+		int spaceLen = 0;
+		int emptyLines = 0;
 
 		for (int i = 0; i < lengthDoc; i++) {
-			char ch = acc[i] ;
-			int style = acc.StyleAt(i) ;
+			char ch = acc[i];
+			int style = acc.StyleAt(i);
 			if (style != styleCurrent) {
-				styleCurrent = style ;
-                styleNew = style ;
+				styleCurrent = style;
+				styleNew = style;
 			}
-            if (ch == ' ') {
-                spaceLen++ ;
-            } else if (ch == '\t') {
-                int ts = tabSize - (lineIndex % tabSize) ;
-                lineIndex += ts - 1 ;
-                spaceLen += ts ;
-            } else if (ch == '\f') {
-                // ignore this animal
-            } else if (ch == '\r' || ch == '\n') {
+			if (ch == ' ') {
+				spaceLen++;
+			} else if (ch == '\t') {
+				int ts = tabSize - (lineIndex % tabSize);
+				lineIndex += ts - 1;
+				spaceLen += ts;
+			} else if (ch == '\f') {
+				// ignore this animal
+			} else if (ch == '\r' || ch == '\n') {
 				if (ch == '\r' && acc[i + 1] == '\n') {
 					i++;
 				}
-                if (styleDone) {
-                    fputs("</t>", fp) ;
-                    styleDone = false ;
-                }
-                lineIndex = -1 ;
-                if (lineDone) {
-                    fputs("</line>\n", fp) ;
-                    lineDone = false ;
-                } else if (collapseLines) {
-                    emptyLines++ ;
-                } else {
-                    fprintf(fp, "<line n='%d'/>\n", lineNumber) ;
-                }
-                charDone = false ;
-                lineNumber++ ;
-                styleCurrent = -1 ; // acc.StyleAt(i + 1) ;
-            } else {
-                if (collapseLines && (emptyLines > 0)) {
-                    fputs("<line/>\n", fp) ;
-                }
-                emptyLines = 0 ;
-                if (! lineDone) {
-                    fprintf(fp, "<line n='%d'>", lineNumber) ;
-                    lineDone = true ;
-                }
-                if (styleNew >= 0) {
-                    if (styleDone) { fputs("</t>", fp) ; }
-                }
-                if (! collapseSpaces) {
-                    while (spaceLen > 0) {
-                        fputs("<s/>", fp) ;
-                        spaceLen-- ;
-                    }
-                } else if (spaceLen == 1) {
-                    fputs("<s/>", fp) ;
-                    spaceLen = 0 ;
-                } else if (spaceLen > 1) {
-                    fprintf(fp, "<s n='%d'/>", spaceLen) ;
-                    spaceLen = 0 ;
-                }
-                if (styleNew >= 0) {
-                    fprintf(fp, "<t n='%d'>", style) ;
-                    styleNew = -1 ;
-                    styleDone = true ;
-                }
-                switch (ch) {
+				if (styleDone) {
+					fputs("</t>", fp);
+					styleDone = false;
+				}
+				lineIndex = -1;
+				if (lineDone) {
+					fputs("</line>\n", fp);
+					lineDone = false;
+				} else if (collapseLines) {
+					emptyLines++;
+				} else {
+					fprintf(fp, "<line n='%d'/>\n", lineNumber);
+				}
+				charDone = false;
+				lineNumber++;
+				styleCurrent = -1; // acc.StyleAt(i + 1);
+			} else {
+				if (collapseLines && (emptyLines > 0)) {
+					fputs("<line/>\n", fp);
+				}
+				emptyLines = 0;
+				if (! lineDone) {
+					fprintf(fp, "<line n='%d'>", lineNumber);
+					lineDone = true;
+				}
+				if (styleNew >= 0) {
+					if (styleDone) { fputs("</t>", fp); }
+				}
+				if (! collapseSpaces) {
+					while (spaceLen > 0) {
+						fputs("<s/>", fp);
+						spaceLen--;
+					}
+				} else if (spaceLen == 1) {
+					fputs("<s/>", fp);
+					spaceLen = 0;
+				} else if (spaceLen > 1) {
+					fprintf(fp, "<s n='%d'/>", spaceLen);
+					spaceLen = 0;
+				}
+				if (styleNew >= 0) {
+					fprintf(fp, "<t n='%d'>", style);
+					styleNew = -1;
+					styleDone = true;
+				}
+				switch (ch) {
 				case '>' :
-					fputs("<g/>", fp) ;
-					break ;
+					fputs("<g/>", fp);
+					break;
 				case '<' :
-					fputs("<l/>", fp) ;
-					break ;
+					fputs("<l/>", fp);
+					break;
 				case '&' :
-					fputs("<a/>", fp) ;
-					break ;
+					fputs("<a/>", fp);
+					break;
 				case '#' :
-					fputs("<h/>", fp) ;
-					break ;
+					fputs("<h/>", fp);
+					break;
 				default  :
-					fputc(ch, fp) ;
-                }
-                charDone = true ;
-            }
-			lineIndex++ ;
+					fputc(ch, fp);
+				}
+				charDone = true;
+			}
+			lineIndex++;
+		}
+		if (styleDone) {
+			fputs("</t>", fp);
+		}
+		if (lineDone) {
+			fputs("</line>\n", fp);
 		}
-        if (styleDone) {
-            fputs("</t>", fp) ;
-        }
-        if (lineDone) {
-            fputs("</line>\n", fp) ;
-        }
 		if (charDone) {
-            // no last empty line: fprintf(fp, "<line n='%d'/>", lineNumber) ;
-        }
+			// no last empty line: fprintf(fp, "<line n='%d'/>", lineNumber);
+		}
 
-		fputs("</text>\n", fp) ;
-		fputs("</document>\n", fp) ;
+		fputs("</text>\n", fp);
+		fputs("</document>\n", fp);
 
-		fclose(fp) ;
+		fclose(fp);
 	} else {
-		SString msg = LocaliseMessage("Could not save file \"^0\".", filePath.AsFileSystem()) ;
-		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING) ;
+		SString msg = LocaliseMessage("Could not save file \"^0\".", filePath.AsFileSystem());
+		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
 	}
 }
