@@ -44,7 +44,7 @@
 #define IDM_ENCODING_UCOOKIE	154
 
 #define MRU_START	16
-#define IMPORT_START 19
+#define IMPORT_START 20
 #define TOOLS_START 3
 
 // Edit
@@ -163,6 +163,7 @@
 #define IDM_OPENGLOBALPROPERTIES	462
 #define IDM_OPENABBREVPROPERTIES	463
 #define IDM_OPENLUAEXTERNALFILE	464
+#define IDM_OPENDIRECTORYPROPERTIES	465
 
 //#define IDM_SELECTIONMARGIN	490
 //#define IDM_BUFFEREDDRAW	491
@@ -174,6 +175,10 @@
 #define IDM_CLOSEALL			503
 #define IDM_SAVEALL			504
 #define IDM_BUFFERSEP		505
+#define IDM_PREVFILESTACK			506
+#define IDM_NEXTFILESTACK			507
+#define IDM_MOVETABRIGHT			508
+#define IDM_MOVETABLEFT			509
 
 // Help
 #define IDM_HELP			901
@@ -226,6 +231,7 @@
 #define IDDOTDOT			249
 #define IDFINDINSTYLE		250
 #define IDFINDSTYLE			251
+#define IDCONVERT			252
 
 #define IDPARAMSTART		300
 
