@@ -448,6 +448,7 @@ protected:
 	enum { toolMax = 50 };
 	Extension *extender;
 	bool needReadProperties;
+	bool allowAlpha;
 
 	int heightOutput;
 	int heightOutputStartDrag;
@@ -757,6 +758,7 @@ protected:
 	virtual void ShowToolBar() = 0;
 	virtual void ShowTabBar() = 0;
 	virtual void ShowStatusBar() = 0;
+	virtual void ActivateWindow(const char *timestamp) = 0;
 
 	void RemoveFindMarks();
 	int MarkAll();
