@@ -1205,11 +1205,11 @@ void SciTEBase::ToolsMenu(int item) {
 						jobType = jobGUI;
 					else if (colon[0] == '2' || 0 == strcmp(colon, "shellexec"))
 						jobType = jobShell;
-					else if (colon[0] == '3' || 0 == strcmp(colon, "lua") || strcmp(colon, "director"))
+					else if (colon[0] == '3' || 0 == strcmp(colon, "lua") || 0 == strcmp(colon, "director"))
 						jobType = jobExtension;
-					else if (colon[0] == '4' || 0 == strcmp(colon, "winhelp"))
+					else if (colon[0] == '4' || 0 == strcmp(colon, "htmlhelp"))
 						jobType = jobHelp;
-					else if (colon[0] == '5' || 0 == strcmp(colon, "htmlhelp"))
+					else if (colon[0] == '5' || 0 == strcmp(colon, "winhelp"))
 						jobType = jobOtherHelp;
 				}
 
