@@ -6,9 +6,13 @@
 // The License.txt file describes the conditions under which this software may be distributed.
 
 class DirectorExtension : public Extension {
+private:
+	DirectorExtension() {} // Singleton
+	DirectorExtension(const DirectorExtension &); // Disable copy ctor
+	void operator=(const DirectorExtension &);    // Disable operator=
+
 public:
-	DirectorExtension();
-	virtual ~DirectorExtension();
+	static DirectorExtension &Instance();
 
 	// Implement the Extension interface
 	virtual bool Initialise(ExtensionAPI *host_);
@@ -28,8 +32,9 @@ public:
 	virtual bool OnUpdateUI();
 	virtual bool OnMarginClick();
 	virtual bool OnMacro(const char *command, const char *params);
-	
+
 	virtual bool SendProperty(const char *prop);
+	virtual bool OnClose(const char *path);
 
 	// Allow messages through to extension
 	void HandleStringMessage(const char *message);
